/*
      *****************************************************************************
      *  Copyright (C)  by Reinhard Heuberger                                     *
      *                                                                           *
      *  All rights reserved.                                                     *
      *                                                                           *
      *  Redistribution and use in source and binary forms, with or without       *
      *  modification, are permitted provided that the following conditions       *
      *  are met:                                                                 *
      *                                                                           *
      *  1. Redistributions of source code must retain the above copyright        *
      *     notice, this list of conditions and the following disclaimer.         *
      *  2. Redistributions in binary form must reproduce the above copyright     *
      *     notice, this list of conditions and the following disclaimer in the   *
      *     documentation and/or other materials provided with the distribution.  *
      *  3. Neither the name of the author nor the names of its contributors may  *
      *     be used to endorse or promote products derived from this software     *
      *     without specific prior written permission.                            *
      *                                                                           *
      *****************************************************************************
      
      RK05 disk drive  Emulator, FM Decoder
*/  
//
module DEcoder (
        Enable,
        CLK_FPGA,                       // System Clock from FPGA
        FM_in,                          // Data
        write_gate,                     // 
        write_protect,
        //
        fm_puls,
        data_serial,                    // Serial Data out
        CLK_8bit,
		  write_gate_out,
        data_byte_out);                 // Data, 8 Bit parallel out
// 
input  Enable, CLK_FPGA, FM_in;
input  write_gate, write_protect;
output fm_puls, data_serial, CLK_8bit, write_gate_out;
output [7:0] data_byte_out;
//
reg fm_p, CLK_8bit_P; 
reg [7:0]  latch_byte;                  // Parallel out
reg [31:0] scan_counter;                // counter, scanning FM
reg [8:0] shifter;                      // shifter, serial->parallel
reg [31:0] byte_clock;
reg data_serial;
wire load_byte;
//
//
initial
begin
  scan_counter    <= 0;
  byte_clock      <= 0;
  shifter         <= 0;
end
//
//
//========================= FM start  =============================
//============================================================================
always @ (posedge CLK_FPGA)
begin
    fm_p <= FM_in;
end
assign fm_puls = FM_in & !fm_p;
assign write_gate_out = write_gate;
//
//=========================== DECODING =======================================
//      
//   _______|-------|_______|-------|_______|-------|_...
//          |<----- 1 ----->|
//   _______|-------|_______________________|-------|_...
//          |<----- 0 --------------------->|<== scan_counter > 25
//          |
//          |<--fm_puls     |<--fm_puls     |<--fm_puls
//          |
//          |<------------->|<== scan_counter < 20
//
//
always @ (posedge CLK_FPGA)
begin
    //
    scan_counter <= scan_counter + 1;
    //
    if(fm_puls) begin
        if ( scan_counter < 20 ) begin
            data_serial <= 1;
			shifter   <= { shifter[7:0] , data_serial};
			byte_clock <= byte_clock + 1;                  // increment byte-counter
        end else if ( scan_counter > 25 )begin
            data_serial <= 0;
			shifter   <= { shifter[7:0] , data_serial};
			byte_clock <= byte_clock + 1;                  // increment byte-counter
            scan_counter <= 0;          
        end
    end 
end
assign CLK_8bit  = ~byte_clock[2];
//
//
//
//
//=============== Generate load pulse for parallel out =======================
//============================================================================
always @ (posedge CLK_FPGA)
begin
 CLK_8bit_P <= CLK_8bit;
end
assign load_byte = CLK_8bit & !CLK_8bit_P;
//
//=========================== latch 8 bit output =============================
//============================================================================
always @ (posedge CLK_FPGA)
begin
  if (load_byte) begin
	latch_byte[7:0] <= shifter[7:0];                 // latch output
  end
end
assign data_byte_out = latch_byte;                          // 8bit, = byte  out
//
//
endmodule