/*
      *****************************************************************************
      *  Copyright (C)  by Reinhard Heuberger                                     *
      *                                                                           *
      *  All rights reserved.                                                     *
      *                                                                           *
      *  Redistribution and use in source and binary forms, with or without       *
      *  modification, are permitted provided that the following conditions       *
      *  are met:                                                                 *
      *                                                                           *
      *  1. Redistributions of source code must retain the above copyright        *
      *     notice, this list of conditions and the following disclaimer.         *
      *  2. Redistributions in binary form must reproduce the above copyright     *
      *     notice, this list of conditions and the following disclaimer in the   *
      *     documentation and/or other materials provided with the distribution.  *
      *  3. Neither the name of the author nor the names of its contributors may  *
      *     be used to endorse or promote products derived from this software     *
      *     without specific prior written permission.                            *
      *                                                                           *
      *****************************************************************************
*/  

//
//
//
module Sequencer ( 
       enable,                 // Enable Input
       CLK_8Bit,               // MFM-ENcoder 16Bit-clock
       sector,                 // Sector pulse-clock
       CLK_FPGA,               // System Clock 65.6 MHz
                               //
       address_out,            // Calculated address of data
       data_end);              // Data End indicator
//   
input enable, CLK_8Bit, sector, CLK_FPGA;
output [13:0] address_out;
output data_end;
//
reg [31:0] current_address;   // augenblickliche Addresse^
reg [31:0] grenze_unten;      // Start Addresse ( DP RAM )
reg [31:0] grenze_oben;       //  bis gültige Obergrenze.
reg [31:0] sector_counter;    // Sector counter
reg [13:0] address_out;       // 14-Bit output Adress to Dual-ported Ram
reg [3:0]  synced_SEC_clk;    // like 2 D-FF register, sync to 50MHz clk.
reg [3:0]  synced_CLK_8Bit;   // like 2 D-FF register, sync to 50MHz clk.
reg data_end;                 // date transmition end_L in one sector
//
initial
begin
   current_address  = 0;
   grenze_unten     = 0;
   grenze_oben      = 544;
end
//
//
//======== Synchronize 8Bit clock, = CLK_8Bit clock to FPGA clock ============
//============================================================================
always @ (posedge CLK_FPGA)
begin
  synced_CLK_8Bit <= { synced_CLK_8Bit[2:0] , CLK_8Bit };   // shift
end
//
//
//======== Synchronize sector clock, = SEC_clk to FPGA clock =================
//============================================================================
always @ (posedge CLK_FPGA)
begin
  synced_SEC_clk <= { synced_SEC_clk[2:0] , sector };   // shift..!..
end
//
//
//==============12 Sector Idex-Adressen bilden, 0-11 ========================
//========= 16Bit Adressen innerhalb eines Sector bilden, 0-139 =============
//===========================================================================
always @ (posedge CLK_FPGA)
begin
   if ( ~synced_SEC_clk[1] & synced_SEC_clk[2] )      // @ negedge SEC clk
   //if ( synced_SEC_clk[1] & ~synced_SEC_clk[2] )      // @ posedge SEC clk
    begin
     data_end <= 1;
     if(enable) begin
      sector_counter = sector_counter +1;          // Increment
       if ( sector_counter == 12 ) begin           //  If Limit ( 0-11 )       
          sector_counter  = 0;                     //      clear counter    
          current_address = 0;                     //      and address
          grenze_unten    = 0;
          grenze_oben     = 544;                   //  set upper limit 
       end else if( sector_counter < 12 ) begin
          grenze_unten = grenze_unten + 544;       //  DP-RAM-Index =
          current_address = grenze_unten;          //  Neue untere Grenze
          grenze_oben = grenze_unten + 544;        //  set upper limit  
       end    
     end   
   end
   //
   // Synchron zur 50Mhz clock werden nun mit der negativen Flanke der
   // 8Bit clock die Adressen innerhalb eines Sectors gebildet.
   //      
   if ( ~synced_CLK_8Bit[1] & synced_CLK_8Bit[2] )      // @ negedge ENC clk
   //if ( synced_CLK_8Bit[1] & ~synced_CLK_8Bit[2] )    // @ posedge ENC clk
    begin                                               // *start*
     if(enable) begin
       if ( current_address == grenze_oben+2) begin 
         data_end <= 0;                               // All done=data_end_L
        end else begin
         current_address = current_address + 1;       // words/sector-counter
         address_out = current_address[13:0];         // send Adress 
         data_end <= 1;              
       end
     end else begin
       current_address = 0;
       grenze_unten    = 0;
       grenze_oben     = 544;
       data_end       <= 0; 
     end
   end    
end
//
endmodule
