/*
              *****************************************************************************
              *  Copyright (C) by Reinhard Heuberger                                      *
              *                                                                           *
              *  All rights reserved.                                                     *
              *                                                                           *
              *  Redistribution and use in source and binary forms, with or without       * 
              *  modification, are permitted provided that the following conditions       * 
              *  are met:                                                                 *
              *                                                                           * 
              *  1. Redistributions of source code must retain the above copyright        * 
              *     notice, this list of conditions and the following disclaimer.         *
              *  2. Redistributions in binary form must reproduce the above copyright     *
              *     notice, this list of conditions and the following disclaimer in the   *
              *     documentation and/or other materials provided with the distribution.  *
              *  3. Neither the name of the author nor the names of its contributors may  * 
              *     be used to endorse or promote products derived from this software     *
              *     without specific prior written permission.                            *
              *                                                                           *
              *****************************************************************************
*/
// This module generates the RK05 sector pulse and index pulse
//
//
module sector_pulse_RK05 ( 
                        CLK_FPGA,
                        base_clk,
                        sector_pulse_H,
                        index_pulse_H,
								secA0,
								secA1,
								secA2,
								secA3);
//
//
input  CLK_FPGA;                   // 46.08 MHz
output base_clk;
output sector_pulse_H, index_pulse_H;
output secA0, secA1, secA2, secA3; 
//
reg baseline, flip;
reg base_clk; 
reg [31:0] count_base;
reg [31:0] count_sector;
reg [31:0] count_index;
reg [31:0] sectorpertrack;
//
reg puls_a, puls_b;
reg mypuls_a, mypuls_b;
wire pa, pb;
//
//
initial
begin
   count_index    <= 0;
   count_sector   <= 0;
   sectorpertrack <= 0;
end
//
//
// internal : Construct the baseline sector clock, 1:1 duty-cyle. 
// FPGA-Clock = 23.04 MHz = 0.0434 uSec 
// Sector-Frequence = 3.3 ms, also 2mal 1.65ms
//  
// (Also 1.65 ms, bzw. 1650 / 0.0434 = 38018
// (Counter läuft aber von 0, also counter-wert 38019
// ( = alte Version 
//
// Also 3.30 ms, bzw. 3300 / 0.0434 = 152073 (76036@23.04 MHz)
// Counter läuft aber von 0, also counter-wert 152074 
//
always @ ( posedge CLK_FPGA ) begin
    if ( count_base <= 152074 ) begin
       count_base = count_base + 1;
    end else begin
       count_base = 0;
       flip = ~flip;
    end
end
always @ ( posedge CLK_FPGA ) 
begin
     base_clk <= ~flip;
end
//
// 
// Construct sector pulse and index pulse
//
// a) sector pulse based on :
//   (20.0uSec/0.0217) = 922 Counter-wert.
always @ ( posedge CLK_FPGA ) begin
    if( count_base) begin
        if( count_sector <= 922 ) begin           // 
            count_sector = count_sector + 1;
            mypuls_a <=1;
        end else begin
            //count_sector = count_sector;
            mypuls_a <=0;
        end
    end else if (!count_base) begin
        count_sector <= 0;
        mypuls_a <=0;
    end
end
assign pa = mypuls_a;
//
//
// b) index pulse, after every 11th sector: 
//  
always @ ( posedge CLK_FPGA ) begin
    if( count_base) begin
        if( count_index <= 1800 ) begin           
            count_index = count_index + 1;
            if( sectorpertrack >= 11) begin 
                mypuls_b <=1;
            end else begin
                mypuls_b <=0;
            end
        end else begin
            mypuls_b <=0;
            if( sectorpertrack >= 11) begin
                sectorpertrack <= 0;
            end
        end
    end else if (!count_base) begin
        sectorpertrack <= sectorpertrack +1;
        count_index <= 0;
        mypuls_b <=0;
    end
end
assign pb = mypuls_b;
assign secA0 = count_index[0];
assign secA1 = count_index[1];
assign secA2 = count_index[2];
assign secA3 = count_index[3]; 
//
//
// synchronizee to system clock
always @ ( posedge CLK_FPGA ) begin
    puls_a <= pa;
end
assign sector_pulse_H =  puls_a;
//
always @ ( posedge CLK_FPGA ) begin
    puls_b <= pb;
end
assign index_pulse_H = puls_b;
//
//
endmodule